#!/usr/bin/env php
<?php
/**
 * Copyright © 2017 Robertas Dereskevicius <info@softdb.eu>, Inc. All rights reserved.
 * See COPYING.txt for license details.
 */

$storeId = 19;
$fastUpdate = true;

if (PHP_SAPI !== 'cli') {
    echo 'bin/magento must be run as a CLI application';
    exit(1);
}

try {
    require __DIR__ . '/../app/bootstrap.php';
} catch (\Exception $e) {
    echo 'Autoload error: ' . $e->getMessage();
    exit(1);
}

class ImportHelper extends \Magento\Framework\App\Helper\AbstractHelper
{
    /**
     * @var \Magento\Catalog\Api\ProductAttributeRepositoryInterface
     */
    protected $attributeRepository;

    /**
     * @var array
     */
    protected $attributeValues;

    /**
     * @var \Magento\Eav\Model\Entity\Attribute\Source\TableFactory
     */
    protected $tableFactory;

    /**
     * @var \Magento\Eav\Api\AttributeOptionManagementInterface
     */
    protected $attributeOptionManagement;

    /**
     * @var \Magento\Eav\Api\Data\AttributeOptionLabelInterfaceFactory
     */
    protected $optionLabelFactory;

    /**
     * @var \Magento\Eav\Api\Data\AttributeOptionInterfaceFactory
     */
    protected $optionFactory;

    /**
     * Data constructor.
     *
     * @param \Magento\Framework\App\Helper\Context $context
     * @param \Magento\Catalog\Api\ProductAttributeRepositoryInterface $attributeRepository
     * @param \Magento\Eav\Model\Entity\Attribute\Source\TableFactory $tableFactory
     * @param \Magento\Eav\Api\AttributeOptionManagementInterface $attributeOptionManagement
     * @param \Magento\Eav\Api\Data\AttributeOptionLabelInterfaceFactory $optionLabelFactory
     * @param \Magento\Eav\Api\Data\AttributeOptionInterfaceFactory $optionFactory
     */
    public function __construct(
        \Magento\Framework\App\Helper\Context $context,
        \Magento\Catalog\Api\ProductAttributeRepositoryInterface $attributeRepository,
        \Magento\Eav\Model\Entity\Attribute\Source\TableFactory $tableFactory,
        \Magento\Eav\Api\AttributeOptionManagementInterface $attributeOptionManagement,
        \Magento\Eav\Api\Data\AttributeOptionLabelInterfaceFactory $optionLabelFactory,
        \Magento\Eav\Api\Data\AttributeOptionInterfaceFactory $optionFactory
    ) {
        parent::__construct($context);

        $this->attributeRepository = $attributeRepository;
        $this->tableFactory = $tableFactory;
        $this->attributeOptionManagement = $attributeOptionManagement;
        $this->optionLabelFactory = $optionLabelFactory;
        $this->optionFactory = $optionFactory;
    }

    /**
     * Get attribute by code.
     *
     * @param string $attributeCode
     * @return \Magento\Catalog\Api\Data\ProductAttributeInterface
     */
    public function getAttribute($attributeCode)
    {
        return $this->attributeRepository->get($attributeCode);
    }

    /**
     * Find or create a matching attribute option
     *
     * @param string $attributeCode Attribute the option should exist in
     * @param string $label Label to find or add
     * @return int
     * @throws \Magento\Framework\Exception\LocalizedException
     */
    public function createOrGetId($attributeCode, $label)
    {
        if (strlen($label) < 1) {
            throw new \Magento\Framework\Exception\LocalizedException(
                __('Label for %1 must not be empty.', $attributeCode)
            );
        }

        // Does it already exist?
        $optionId = $this->getOptionId($attributeCode, $label);

        if (!$optionId) {
            // If no, add it.

            /** @var \Magento\Eav\Model\Entity\Attribute\OptionLabel $optionLabel */
            $optionLabel = $this->optionLabelFactory->create();
            $optionLabel->setStoreId(0);
            $optionLabel->setLabel($label);

            $option = $this->optionFactory->create();
            $option->setLabel($optionLabel);
            $option->setStoreLabels([$optionLabel]);
            $option->setSortOrder(0);
            $option->setIsDefault(false);

            $this->attributeOptionManagement->add(
                \Magento\Catalog\Model\Product::ENTITY,
                $this->getAttribute($attributeCode)->getAttributeId(),
                $option
            );

            // Get the inserted ID. Should be returned from the installer, but it isn't.
            $optionId = $this->getOptionId($attributeCode, $label, true);
        }

        return $optionId;
    }

    /**
     * Find the ID of an option matching $label, if any.
     *
     * @param string $attributeCode Attribute code
     * @param string $label Label to find
     * @param bool $force If true, will fetch the options even if they're already cached.
     * @return int|false
     */
    public function getOptionId($attributeCode, $label, $force = false)
    {
        /** @var \Magento\Catalog\Model\ResourceModel\Eav\Attribute $attribute */
        $attribute = $this->getAttribute($attributeCode);

        // Build option array if necessary
        if ($force === true || !isset($this->attributeValues[ $attribute->getAttributeId() ])) {
            $this->attributeValues[ $attribute->getAttributeId() ] = [];

            // We have to generate a new sourceModel instance each time through to prevent it from
            // referencing its _options cache. No other way to get it to pick up newly-added values.

            /** @var \Magento\Eav\Model\Entity\Attribute\Source\Table $sourceModel */
            $sourceModel = $this->tableFactory->create();
            $sourceModel->setAttribute($attribute);

            foreach ($sourceModel->getAllOptions() as $option) {
                $this->attributeValues[ $attribute->getAttributeId() ][ $option['label'] ] = $option['value'];
            }
        }

        // Return option ID if exists
        if (isset($this->attributeValues[ $attribute->getAttributeId() ][ $label ])) {
            return $this->attributeValues[ $attribute->getAttributeId() ][ $label ];
        }

        // Return false if does not exist
        return false;
    }
}
	require (dirname(__FILE__).'/../lib/PHPExcel.php');
	require (dirname(__FILE__).'/../lib/PHPExcel/Writer/Excel2007.php');
	require_once (dirname(__FILE__).'/../lib/PHPExcel/IOFactory.php');

	if (count($_SERVER["argv"]) != 2 || !file_exists($_SERVER["argv"][1]))
	    die("Import File not found..\n");


	$inputFileType = 'Excel5';
	$inputFileName = $_SERVER["argv"][1];
	$tmpFile = dirname(__FILE__).'/../var/tmp/exportFile.csv';

	$objReader = PHPExcel_IOFactory::createReader($inputFileType);
	$objPHPExcel = @$objReader->load($inputFileName);

	$objWriter = PHPExcel_IOFactory::createWriter($objPHPExcel, 'CSV');
	$objWriter->save($tmpFile);

	echo "created csv file ".basename($tmpFile)."\n";

	$categories = "";
	if (file_exists(dirname(__FILE__)."/categories.csv")){
	    if (($handle = fopen(dirname(__FILE__)."/categories.csv", "r")) !== FALSE) {
	        while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
		    $categories[$data[0]]=$data[1];
		}
	    }
	}
//print "<pre>";print_r($categories);print count($categories);exit;
	
	$bootstrap = \Magento\Framework\App\Bootstrap::create(BP, $_SERVER);
	/** @var \Magento\Framework\ObjectManager\ObjectManager $objectManager */
	$objectManager = $bootstrap->getObjectManager();
	$objectManager->get('\Magento\Framework\App\State')->setAreaCode('frontend');

        $resourceConnection = $objectManager->create('\Magento\Framework\App\ResourceConnection');
        $connection = $resourceConnection->getConnection();

	$tblProduct = $connection->getTableName('catalog_product_entity');
	$tblProductDecimal = $connection->getTableName('catalog_product_entity_decimal');
	$tblProductCategory = $connection->getTableName('catalog_category_product');
	$tblProductCategoryIndex = $connection->getTableName('catalog_category_product_index');
	$tblUrlRewrite = $connection->getTableName('url_rewrite');
	$tblCategory = $connection->getTableName('catalog_category_entity');

	foreach ($categories as $k=>$v) {
		if (!$v) continue;
        $query = "select entity_id from ".$tblCategory." where entity_id='$v'";
		$entity_id = $connection->fetchOne($query);
		if (!$entity_id) die("Category $k with id $v not found\n");
	}

	$eavAttribute = $objectManager->get("\Magento\Eav\Model\ResourceModel\Entity\Attribute");
	$attributeIdBarcode = $eavAttribute->getIdByCode('catalog_product', 'barcode');
	if (!$attributeIdBarcode) die ("Product attribute 'barcode' not found\n");
	$attributeIdBrand = $eavAttribute->getIdByCode('catalog_product', 'brand');
	if (!$attributeIdBarcode) die ("Product attribute 'brand' not found\n");
	$attributeIdSupplier = $eavAttribute->getIdByCode('catalog_product', 'supplier');
	if (!$attributeIdSupplier) die ("Product attribute 'supplier' not found\n");
	$attributeIdPrice = $eavAttribute->getIdByCode('catalog_product', 'price');
	$attributeIdCost = $eavAttribute->getIdByCode('catalog_product', 'cost');

$row = $updated = $inserted = 0;
$header = array();
$brand = $supplier = $category1 = $category2 ="";


$context = $objectManager->get("\Magento\Framework\App\Helper\Context");
$productAttributeRepository = $objectManager->get("Magento\Catalog\Api\ProductAttributeRepositoryInterface");
$tableFactory = $objectManager->get("\Magento\Eav\Model\Entity\Attribute\Source\TableFactory");
$attributeOptionManagement = $objectManager->get("\Magento\Eav\Api\AttributeOptionManagementInterface");
$optionLabelFactory = $objectManager->get("\Magento\Eav\Api\Data\AttributeOptionLabelInterfaceFactory");
$optionFactory = $objectManager->get("\Magento\Eav\Api\Data\AttributeOptionInterfaceFactory");
$helper = new ImportHelper($context,$productAttributeRepository,$tableFactory,$attributeOptionManagement,$optionLabelFactory,$optionFactory);
$productFactory = $objectManager->get('\Magento\Catalog\Model\ProductFactory');

if (($handle = fopen($tmpFile, "r")) !== FALSE) {
    while (($data = fgetcsv($handle, 1000, ",")) !== FALSE) {
        $num = count($data);
        $row++;
	if ($row == 1) {
	    foreach ($data as $k=>$v) {
		$header[$k] = strtolower($v);
		$header[strtolower($v)] = $k;
	    }
	    continue;
	}
	//if ($row<10000) continue;
/*
    [0] => lteam
    [lteam] => 0
    [1] => department name
    [department name] => 1
    [2] => main category
    [main category] => 2
    [3] => category
    [category] => 3
    [4] => brand name
    [brand name] => 4
    [5] => supplier name
    [supplier name] => 5
    [6] => item id
    [item id] => 6
    [7] => barcode
    [barcode] => 7
    [8] => product name
    [product name] => 8
    [9] => net rate
    [net rate] => 9
    [10] => selling price
    [selling price] => 10
*/

    $sku = trim($data[$header["item id"]]);
    $barcode = trim($data[$header["barcode"]]);
    //if (empty($sku) && !empty($barcode)) $sku = $barcode;
    if (empty($sku)) {
	//echo ("Empty SKU found... ".print_r($data,true));
	continue;
    }
    $barcode = $data[$header["barcode"]];
    $name = trim($data[$header["product name"]]);
    $cost = $data[$header["net rate"]];
    $price = $data[$header["selling price"]];
    if (!empty($data[$header["brand name"]]))
        $brand = $data[$header["brand name"]];
    if ($brand == "Default") {$brand="";$brandid=0;}
    if (!empty($brand))	
	$brandid = $helper->createOrGetId('brand', $brand);
//print "brand:$brand;brandid:$brandid\n";exit;
    if (!empty($data[$header["supplier name"]]))
        $supplier = $data[$header["supplier name"]];
    if (!empty($data[$header["main category"]]))
        $category1 = $data[$header["main category"]];
    $category1 = str_replace(","," ",$category1);
    $category2 = $data[$header["category"]];
    $category2 = str_replace(","," ",$category2);
    $category = strtolower("$category1/$category2");
    if (!isset($categories[$category])) {
	echo "\nCategory $category not found\n";
	$categories[$category]="";
	$fh = fopen(dirname(__FILE__)."/categories.csv","w");
	foreach ($categories as $k=>$v)
	    fwrite($fh,"$k,$v\n");
	fclose($fh);
    }

    
    $query = "select entity_id from ".$tblProduct." where sku='".addslashes($sku)."'";
    $entity_id = $connection->fetchOne($query);

    if (!$entity_id) { //create product
        $url = preg_replace('#[^0-9a-z]+#i', '-', $name);
        $url = preg_replace('/-$/', '', $url);
        $url = strtolower($url);
        $query = "select url_rewrite_id from ".$tblUrlRewrite." where request_path='".addslashes($url)."' or request_path='".addslashes($url).".html'";
	$url_rewrite_id = $connection->fetchOne($query);
	if ($url_rewrite_id) $url.="-".$row;
//print "creating $url  $query\n";//exit;
	$product = $objectManager->create('\Magento\Catalog\Model\Product');
	$product->setSku($sku); // Set your sku here
	$product->setName($name); // Name of Product
	$product->setBarcode($barcode);
	$product->setUrlKey($url);
	$product->setAttributeSetId(4); // Attribute set id
	$product->setStatus(1); // Status on product enabled/ disabled 1/0
	$product->setWeight(0.001); // weight of product
	$product->setVisibility(4); // visibilty of product (catalog / search / catalog, search / Not visible individually)
	$product->setWebsiteIds(array(1));
	$product->setTaxClassId(2); // Tax class id
	$product->setTypeId('simple'); // type of product (simple/virtual/downloadable/configurable)
	$product->setCost($cost); // price of product
	$product->setPrice($price); // price of product
	$product->setData('brand',$brandid);
	$product->setSupplier($supplier);
	$product->setStockData(
                        array(
                            'use_config_manage_stock' => 0,
                            'manage_stock' => 0,
                            'is_in_stock' => 1,
                            'qty' => 100
                        )
                    );
	$categoryid = $categories[$category];
	if (is_numeric($categoryid))
		 $product->setCategoryIds($categoryid); 
	$product->setIsMassupdate(true)->save();
	$inserted++;
	//print $product->getId();exit;
	//break;
    } else {
      if ($fastUpdate) {
	    $storeIdMain = 0;
	    $sql = "INSERT INTO " . $tblProductDecimal . " (attribute_id, store_id, entity_id, value) VALUES (?, ?, ?, ?) ON DUPLICATE KEY UPDATE value=VALUES(value)";
	    $connection->query(
    		$sql,
	        [
    		    $attributeIdPrice,
	            $storeIdMain,
	            $entity_id,
	            $price
	        ]
	    );
	    $connection->query(
    		$sql,
	        [
    		    $attributeIdCost,
	            $storeIdMain,
	            $entity_id,
	            $cost
	        ]
	    );
	    $categoryid = $categories[$category];
	    if (!empty($categoryid) && is_numeric($categoryid)) {
		    $sql = "INSERT IGNORE INTO " . $tblProductCategory . " (category_id, product_id, position) VALUES (?, ?, 1)";
//print "$categoryid,$entity_id $sql\n";exit;
		    $connection->query(
    			$sql,
	    	        [
		            $categoryid,
		            $entity_id
		        ]
		    );
		    $sql = "INSERT IGNORE INTO " . $tblProductCategoryIndex . " (category_id, product_id, store_id, visibility) VALUES (?, ?, ?, 4)";
//print "$categoryid,$entity_id $sql\n";exit;
		    $connection->query(
    			$sql,
	    	        [
		            $categoryid,
		            $entity_id,
			    $storeId
		        ]
		    );
		    $sql = "INSERT IGNORE INTO " . $tblProductCategoryIndex . " (category_id, product_id, store_id, visibility) VALUES (?, ?, ?, 4)";
//print "$categoryid,$entity_id $sql\n";exit;
			if ($storeId != 0)
		    $connection->query(
    			$sql,
	    	        [
		            $categoryid,
		            $entity_id,
					0
		        ]
		    );
	    }
      } else {
	$product = $productFactory->create()->load($entity_id);
	    $categoryid = $categories[$category];
	    if (is_numeric($categoryid))
		 $product->setCategoryIds($categoryid); 
	//$product->setBarcode($barcode);
	//$product->setBrand($brandid);
	$product->setWebsiteIds(array(1));
	//$product->setName($name);
	//$product->setData('brand',$brandid);
	//$product->setSupplier($supplier);
	$product->setCost($cost);
	$product->setPrice($price);
//print "$brand;$brandid\n";exit;
	$product->setIsMassupdate(true)->save();
//print $product->getId();print ";sku:$sku\nbarcode:$barcode\ncost:$cost;price:$price\nbrand:$brand;brandid:$brandid;supplier:$supplier\ncat:$category1;cat2:$category2\n";exit;
      }
	$updated++;
	//break;
    }
if (($row%100) == 0) print ".";
if (($row%1000) == 0) print "\n$row";
    //print "sku:$sku\nbarcode:$barcode\ncost:$cost;price:$price\nbrand:$brand;supplier:$supplier\ncat:$category1;cat2:$category2\n";exit;
    }
    fclose($handle);
}
echo "Created $inserted new product(s). Updated $updated product(s).\n";




